@@ -1,6 +1,7 @@
 import ConfigParser
 import logging
 import re
+import subprocess
 
 from cStringIO import StringIO
 
@@ -9,78 +10,13 @@ from . import exc
 from . import lsb
 from .cliutil import priority
 from .sudo_pushy import get_transport
+from .util import paths
+from . import hosts
 
 
 LOG = logging.getLogger(__name__)
 
 
-def create_mon(cluster, monitor_keyring, init):
-    import os
-    import socket
-    import subprocess
-
-    hostname = socket.gethostname().split('.')[0]
-    path = '/var/lib/ceph/mon/ceph-{hostname}'.format(
-        hostname=hostname,
-        )
-    done_path = '/var/lib/ceph/mon/ceph-{hostname}/done'.format(
-        hostname=hostname,
-        )
-    init_path = '/var/lib/ceph/mon/ceph-{hostname}/{init}'.format(
-        hostname=hostname,
-        init=init,
-        )
-
-    if not os.path.exists(path):
-        os.mkdir(path)
-
-    if not os.path.exists(done_path):
-        keyring = '/var/lib/ceph/tmp/{cluster}-{hostname}.mon.keyring'.format(
-            cluster=cluster,
-            hostname=hostname,
-            )
-
-        with file(keyring, 'w') as f:
-            f.write(monitor_keyring)
-
-        subprocess.check_call(
-            args=[
-                'ceph-mon',
-                '--cluster', cluster,
-                '--mkfs',
-                '-i', hostname,
-                '--keyring', keyring,
-                ],
-            )
-        os.unlink(keyring)
-        with file(done_path, 'w'):
-            pass
-
-    if not os.path.exists(init_path):
-        with file(init_path, 'w'):
-            pass
-
-    if init == 'upstart':
-        subprocess.check_call(
-            args=[
-                'initctl',
-                'emit',
-                'ceph-mon',
-                'cluster={cluster}'.format(cluster=cluster),
-                'id={hostname}'.format(hostname=hostname),
-                ],
-            )
-    elif init == 'sysvinit':
-        subprocess.check_call(
-            args=[
-                'service',
-                'ceph',
-                'start',
-                'mon.{hostname}'.format(hostname=hostname),
-                ],
-            )
-
-
 def mon_create(args):
 
     cfg = conf.load(args)
@@ -112,36 +48,15 @@ def mon_create(args):
     errors = 0
     for hostname in args.mon:
         try:
-            LOG.debug('Deploying mon to %s', hostname)
-
             # TODO username
-            sudo = args.pushy(get_transport(hostname))
-
-            (distro, release, codename) = lsb.get_lsb_release(sudo)
-            init = lsb.choose_init(distro, codename)
-            LOG.debug('Distro %s codename %s, will use %s',
-                      distro, codename, init)
-
-            write_conf_r = sudo.compile(conf.write_conf)
-            conf_data = StringIO()
-            cfg.write(conf_data)
-            write_conf_r(
-                cluster=args.cluster,
-                conf=conf_data.getvalue(),
-                overwrite=args.overwrite_conf,
-                )
-
-            create_mon_r = sudo.compile(create_mon)
-            create_mon_r(
-                cluster=args.cluster,
-                monitor_keyring=monitor_keyring,
-                init=init,
-                )
-
             # TODO add_bootstrap_peer_hint
-
-            sudo.close()
-
+            LOG.debug('detecting platform for host %s ...', hostname)
+            distro = hosts.get(hostname)
+            LOG.info('distro info: %s %s %s', distro.name, distro.release, distro.codename)
+            rlogger = logging.getLogger(hostname)
+            rlogger.debug('deploying mon to %s', hostname)
+            distro.mon.create(distro, rlogger, args, monitor_keyring)
+            distro.sudo_conn.close()
         except RuntimeError as e:
             LOG.error(e)
             errors += 1
@@ -150,19 +65,21 @@ def mon_create(args):
         raise exc.GenericError('Failed to create %d monitors' % errors)
 
 
-def destroy_mon(cluster):
+def destroy_mon(cluster, paths, is_running):
+    import datetime
+    import errno
     import os
-    import subprocess
+    import subprocess  # noqa
     import socket
+    import time
+    retries = 5
 
     hostname = socket.gethostname().split('.')[0]
-    path = '/var/lib/ceph/mon/ceph-{hostname}'.format(
-        hostname=hostname,
-        )
+    path = paths.mon.path(cluster, hostname)
 
     if os.path.exists(path):
         # remove from cluster
-        subprocess.check_call(
+        proc = subprocess.Popen(
             args=[
                 'sudo',
                 'ceph',
@@ -173,38 +90,60 @@ def destroy_mon(cluster):
                 'remove',
                 hostname,
                 ],
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
             )
+        out, err = proc.communicate()
+        return_status = proc.wait()
+        if return_status > 0:
+            raise RuntimeError(err.strip())
 
         # stop
         if os.path.exists(os.path.join(path, 'upstart')):
-            subprocess.call(   # ignore initctl error when job not running
-                args=[
-                    'initctl',
-                    'stop',
-                    'ceph-mon',
-                    'cluster={cluster}'.format(cluster=cluster),
-                    'id={hostname}'.format(hostname=hostname),
-                ],
-            )
-        elif os.path.exists(os.path.join(path, 'sysvinit')):
-            subprocess.check_call(
-                args=[
-                    'service',
-                    'ceph',
-                    'stop',
-                    'mon.{hostname}'.format(hostname=hostname),
-                ],
-            )
+            status_args = [
+                'initctl',
+                'status',
+                'ceph-mon',
+                'cluster={cluster}'.format(cluster=cluster),
+                'id={hostname}'.format(hostname=hostname),
+            ]
 
-        # delete monitor directory
+        elif os.path.exists(os.path.join(path, 'sysvinit')):
+            status_args = [
+                'service',
+                'ceph',
+                'status',
+                'mon.{hostname}'.format(hostname=hostname),
+            ]
+
+        while retries:
+            if is_running(status_args):
+                time.sleep(5)
+                retries -= 1
+                if retries <= 0:
+                    raise RuntimeError('ceph-mon deamon did not stop')
+            else:
+                break
+
+        # archive old monitor directory
+        fn = '{cluster}-{hostname}-{stamp}'.format(
+            hostname=hostname,
+            cluster=cluster,
+            stamp=datetime.datetime.utcnow().strftime("%Y-%m-%dZ%H:%M:%S"),
+            )
         subprocess.check_call(
             args=[
-                'rm',
-                '-rf',
-                path,
+                'mkdir',
+                '-p',
+                '/var/lib/ceph/mon-removed',
                 ],
             )
-
+        try:
+            os.makedirs('/var/lib/ceph/mon-removed')
+        except OSError, e:
+            if e.errno != errno.EEXIST:
+                raise
+        os.rename(path, os.path.join('/var/lib/ceph/mon-removed/', fn))
 
 def mon_destroy(args):
     errors = 0
@@ -218,6 +157,8 @@ def mon_destroy(args):
             destroy_mon_r = sudo.compile(destroy_mon)
             destroy_mon_r(
                 cluster=args.cluster,
+                paths=paths,
+                is_running=is_running,
                 )
             sudo.close()
 
@@ -226,7 +167,7 @@ def mon_destroy(args):
             errors += 1
 
     if errors:
-        raise exc.GenericError('Failed to create %d monitors' % errors)
+        raise exc.GenericError('Failed to destroy %d monitors' % errors)
 
 
 def mon(args):
@@ -237,6 +178,7 @@ def mon(args):
     else:
         LOG.error('subcommand %s not implemented', args.subcommand)
 
+
 @priority(30)
 def make(parser):
     """
@@ -260,3 +202,36 @@ def make(parser):
     parser.set_defaults(
         func=mon,
         )
+
+#
+# Helpers
+#
+
+
+def is_running(args):
+    """
+    Run a command to check the status of a mon, return a boolean.
+
+    We heavily depend on the format of the output, if that ever changes
+    we need to modify this.
+    Check daemon status for 3 times
+    output of the status should be similar to::
+
+        mon.mira094: running {"version":"0.61.5"}
+
+    or when it fails::
+
+        mon.mira094: dead {"version":"0.61.5"}
+        mon.mira094: not running {"version":"0.61.5"}
+    """
+    proc = subprocess.Popen(
+        args=args,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+    )
+    result = proc.communicate()
+    result_string = ' '.join(result)
+    for run_check in [': running', ' start/running']:
+        if run_check in result_string:
+            return True
+    return False
